#include <malloc.h>
#include <stdio.h>
#include <string.h>

typedef struct _THESTRUCT
{	int     maxnum;			// how many pointers are available
	int     count;			// how many have we used
	char * 	str[];			// pointers to pointers of char (*str[])
}THESTRUCT;

THESTRUCT * init_thestruct( int num )
{
	// Allocate memory for the structure which includes
	// num pointers to char *[]
	THESTRUCT * ts = malloc(sizeof(THESTRUCT) + sizeof(char**) * num );
	if(ts){

		// Set the number of possible pointers
		ts->maxnum = num;

		// no strings as yet
		ts->count  = 0;
		return ts;

	}else{
		return NULL;
	}
}

void free_struct( THESTRUCT * ts )
{
	// free all memory accoiated with our structure
	if(ts){
		for(int i = 0; i<ts->count; i++)
			free(ts->str[i]);
		free(ts);
	}

}

THESTRUCT * realloc_struct( THESTRUCT * ts )
{
	int num = ts->maxnum;

	// Double the num of pointers that the structure will hold
	num *= 2;

	// Allocate the new memory for the structure and pointers
	THESTRUCT * tmp = realloc(ts, sizeof(THESTRUCT) + sizeof(char**) * num );
	if(!tmp){
		return NULL;
	}else{

		// Set new maxnum pointers and new struct pointer
		ts         = tmp;
		ts->maxnum = num;

		return ts;
	}
}

THESTRUCT * add_string( THESTRUCT * ts, char * str )
{
	THESTRUCT * tmp;

	// Allocate enough memory for the string + 1
	char * tmpc = malloc(strlen(str)+1);
	if(!tmpc){
		return NULL; // no mem
	}else{

		// Check, are there enough pointers,
		// if not go make some more
		if(ts->count == ts->maxnum){
			tmp = realloc_struct(ts);

			if(tmp == NULL)
				return NULL;
			else
				ts = tmp;
		}

		// Make the next pointer contain the new string memory pointer
		ts->str[ts->count] = tmpc;

		// copy the string to our newly allocated memory
		strcpy(ts->str[ts->count], str);

		// increment counter for the next time
		ts->count++;
		return ts;
	}
}

int main (void)
{
	int i;
	char s[200];

	// Initialise the structure that will contain our strings
	THESTRUCT * tmp, * ts = init_thestruct(2);
	if(!ts){
		printf("No memory today!\n");
		return 0;
	}

	for(i = 0; i<10; i++){
		sprintf(s, "String %d", i);

		// Add each string - if more pointers are required
		// add_string calls realloc_struct()
		if(NULL != (tmp = add_string(ts, s))){

			// The struct memory may have changed if it was realloc-ed
			// so we need to re-assign our pointer to struct, 'ts'
			ts = tmp;

		}else{

			// oops, no memory!
			break;
		}
	}

	for(i = 0; i<ts->count; i++){
		printf("%s\n", ts->str[i]);
	}
	free_struct(ts);

	return 0;
}

