#include <malloc.h>
#include <stdio.h>
#include <string.h>

typedef struct _THESTRUCT
{	int     maxnum;			// how many pointers are available
	int     count;			// how many have we used
	char ** str;			// pointers to pointers of char (*str[])
}THESTRUCT;

THESTRUCT * init_thestruct( int num )
{
	// Allocate memory for the structure
	THESTRUCT * ts = malloc(sizeof(THESTRUCT));
	if(ts){

		// Set the number of possible pointers
		ts->maxnum = num;

		// no strings as yet
		ts->count  = 0;

		// Allocate num pointers to pointers
		ts->str    = calloc(1, sizeof(char**) * num );

		if(!ts->str){
			free(ts);
			return NULL;
		}else{
			// if succesful return the pointer to struct
			return ts;
		}
	}else{
		return NULL;
	}
}

void free_struct( THESTRUCT * ts )
{
	// free all memory accoiated with our structure
	if(ts){
		if(ts->str){
			for(int i = 0; i<ts->count; i++)
				free(ts->str[i]);
		}
		free(ts->str);
		free(ts);
	}
}

int realloc_pointers( THESTRUCT * ts )
{
	int num = ts->maxnum;

	// Double the num of pointers that the structure will hold
	num *= 2;

	// Allocate the new memory
	char ** tmp = realloc(ts->str, sizeof(char**) * num);
	if(!tmp){
		return -1;
	}else{

		// Set new maxnum pointers
		ts->maxnum = num;

		// Set the new pointers to pointers memory
		ts->str    = tmp;
		return 0;
	}
}

int add_string( THESTRUCT * ts, char * str )
{
	int ret = 0;
	// Allocate enough memory for the string + 1
	char * tmp = malloc(strlen(str)+1);
	if(!tmp){
		return -1; // no mem
	}else{

		// Check, are there enough pointers,
		// if not go make some more
		if(ts->count == ts->maxnum)
			ret = realloc_pointers(ts);

		if(ret == -1)
			return -1;

		// Make the next pointer contain the new string memory pointer
		ts->str[ts->count] = tmp;

		// copy the string to our newly allocated memory
		strcpy(ts->str[ts->count], str);

		// increment counter for the next time
		ts->count++;
		return 0;
	}
}

int main (void)
{
	int i;
	char s[200];

	// Initialise the structure that will contain our strings
	THESTRUCT * ts = init_thestruct(2);
	if(!ts){
		printf("No memory today!\n");
		return 0;
	}

	for(i = 0; i<10; i++){
		sprintf(s, "String %d", i);
		// Add each string - if more pointers are required
		// add_string calls realloc_pointers()
		if(-1 == add_string(ts, s))
			break;
	}
	for(i = 0; i<ts->count; i++){
		printf("%s\n", ts->str[i]);
	}
	free_struct(ts);

	return 0;
}

